import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';

import '../renderer/actor.dart';
import '../renderer/stage.dart';
import 'events.dart';

mixin HasHoverablesMixin on Stage {
  @mustCallSuper
  void onMouseMove(PointerHoverInfo info) {
    propagateToChildren<HoverableMixin>((child) => child.handleMouseMovement(info));
  }

  void onMouseEnter(PointerEnterInfo info) {}

  void onMouseLeave(PointerLeaveInfo info) {}
}

mixin HoverableMixin on Actor {
  bool _isHovered = false;
  bool get isHovered => _isHovered;

  @nonVirtual
  bool handleMouseMovement(PointerHoverInfo info) {
    if (containsPoint(info.position)) {
      if (_isHovered) {
        return onMouseMove(info);
      }

      _isHovered = true;
      final shouldContinue = onMouseEnter(info);
      onMouseMove(info);
      return shouldContinue;
    } else if (_isHovered) {
      _isHovered = false;
      return onMouseLeave(info);
    }

    return true;
  }

  bool onMouseMove(PointerHoverInfo info) => true;
  bool onMouseEnter(PointerHoverInfo info) => true;
  bool onMouseLeave(PointerHoverInfo info) => true;
}

class PointerEnterInfo extends PositionInfo<PointerEnterEvent> with HandledField {
  PointerEnterInfo.fromDetails(
    Stage stage,
    PointerEnterEvent raw,
  ) : super(stage, raw.position, raw);
}

class PointerLeaveInfo extends PositionInfo<PointerExitEvent> with HandledField {
  PointerLeaveInfo.fromDetails(
    Stage stage,
    PointerExitEvent raw,
  ) : super(stage, raw.position, raw);
}

class PointerHoverInfo extends PositionInfo<PointerHoverEvent> with HandledField {
  PointerHoverInfo.fromDetails(
    Stage stage,
    PointerHoverEvent raw,
  ) : super(stage, raw.position, raw);
}
