import 'package:flutter/rendering.dart';
import 'package:flutter/widgets.dart';

import '../renderer/stage.dart';

class StageRenderBox extends RenderBox {
  BuildContext buildContext;
  Stage stage;

  int? _scheduleId;

  StageRenderBox(this.buildContext, this.stage) {
    _scheduler.addTimingsCallback(stage.onTimingsCallback);
  }

  @override
  bool isRepaintBoundary = true;

  @override
  bool sizedByParent = true;

  @override
  Size computeDryLayout(BoxConstraints constraints) => constraints.biggest;

  WidgetsBinding get _scheduler => WidgetsBinding.instance!;

  @override
  void attach(PipelineOwner owner) {
    super.attach(owner);
    stage.attach(owner, this);
  }

  @override
  void detach() {
    super.detach();
    stage.detach();
  }

  @override
  void dispose() {
    super.dispose();
    _scheduler.removeTimingsCallback(stage.onTimingsCallback);
    if (_scheduleId != null) {
      _scheduler.cancelFrameCallbackWithId(_scheduleId!);
    }
  }

  void refresh() {
    _scheduleId ??= _scheduler.scheduleFrameCallback((_) {
      stage.updateTree();
      markNeedsPaint();
      _scheduleId = null;
    });
  }

  @override
  void paint(PaintingContext context, Offset offset) {
    context.canvas
      ..save()
      ..translate(offset.dx, offset.dy);
    stage.renderTree(context.canvas);
    context.canvas.restore();
  }
}
