import 'apply_gestures.dart';
import 'resize_widget.dart';
import 'package:flutter/widgets.dart';
import 'package:vector_math/vector_math_64.dart';

import 'stage_render_box.dart';
import '../renderer/stage.dart';

class TheatreWidget<T extends Stage> extends StatefulWidget {
  final T stage;

  const TheatreWidget({
    Key? key,
    required this.stage,
  }) : super(key: key);

  @override
  _TheatreWidgetState createState() => _TheatreWidgetState();
}

class _TheatreWidgetState extends State<TheatreWidget> {
  int _layoutKey = 0;

  @override
  void initState() {
    super.initState();
    widget.stage.onMount();
  }

  @override
  void dispose() {
    super.dispose();
    widget.stage.onRemove();
  }

  @override
  void didUpdateWidget(covariant TheatreWidget<Stage> oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (oldWidget.stage != widget.stage) {
      ++_layoutKey;
      oldWidget.stage
        ..detach()
        ..onRemove();
    }
  }

  @override
  Widget build(BuildContext context) {
    final stage = widget.stage;

    Widget internalWidget = _StageRenderObjectWidget(stage: widget.stage);
    internalWidget = applyMouseMoveGesturesDetectors(stage, internalWidget);
    internalWidget = applyScrollDetector(stage, internalWidget);
    internalWidget = applyBasicGesturesDetectors(stage, internalWidget);
    internalWidget = applyAdvancedGesturesDetectors(stage, internalWidget);

    final List<Widget> internalWidgets = [internalWidget];

    return LayoutBuilder(
      key: Key(_layoutKey.toString()),
      builder: (_, constraints) {
        final size = constraints.biggest;

        if (size.isEmpty) {
          return const SizedBox.shrink();
        }

        return Stack(
          children: [
            ResizeWidget(
              stage: stage,
              canvasSize: Vector2(size.width, size.height),
            ),
            ...internalWidgets
          ],
        );
      },
    );
  }
}

class _StageRenderObjectWidget extends LeafRenderObjectWidget {
  final Stage stage;

  const _StageRenderObjectWidget({required this.stage});

  @override
  RenderBox createRenderObject(BuildContext context) {
    return StageRenderBox(context, stage);
  }
}
